function [] = ro_kriging_experiments(do_tests, do_analysis)

	stopeval = 5000;
	runs = 30;
	N = 10;

	fitnessfct = @sphere;
	lb = -10 * ones(N,1);
	ub = 10 * ones(N,1);
	delta = .5 * ones(N,1);
	if (do_tests)
		run_test(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end
	if (do_analysis)
		run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end

	fitnessfct = @rotated_sphere;
	lb = -10 * ones(N,1);
	ub = 10 * ones(N,1);
	delta = .5 * ones(N,1);
	if (do_tests)
		run_test(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end
	if (do_analysis)
		run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end

	fitnessfct = @kruisselbrink_tp1;
	lb = -10 * ones(N,1);
	ub = 10 * ones(N,1);
	delta = 2 * ones(N,1);
	if (do_tests)
		run_test(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end
	if (do_analysis)
		run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end

	fitnessfct = @rotated_kruisselbrink_tp1;
	lb = -10 * ones(N,1);
	ub = 10 * ones(N,1);
	delta = 2 * ones(N,1);
	if (do_tests)
		run_test(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end
	if (do_analysis)
		run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end

	fitnessfct = @kruisselbrink_tp2;
	lb = -10 * ones(N,1);
	ub = 10 * ones(N,1);
	delta = 2 * ones(N,1);
	if (do_tests)
		run_test(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end
	if (do_analysis)
		run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end

	fitnessfct = @rotated_kruisselbrink_tp2;
	lb = -10 * ones(N,1);
	ub = 10 * ones(N,1);
	delta = 2 * ones(N,1);
	if (do_tests)
		run_test(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end
	if (do_analysis)
		run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs);
	end

end


function [] = run_test(fitnessfct, N, lb, ub, delta, stopeval, runs)

	disp(['Test function: ' func2str(fitnessfct)])

	algorithms = 4;

	stat.fitnessfct = fitnessfct;
	stat.N = N;
	stat.lb = lb;
	stat.ub = ub;
	stat.delta = delta;
	stat.stopeval = stopeval;
	stat.algorithms = algorithms;
	stat.runs = runs;

	param.doplot = false;
	param.optimizer = @cmaes;
	param.bch = @cutoff_bc;
	epsilon = -1;

	optimizer_stats = [];
	optimizer_name = ['SEM'];
	optimizer_index = 1;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	if (~exist(optimizer_stats_file, 'file'))
		for i = 1 : runs
			disp([optimizer_name, ': test run: ' int2str(i)])
			[xopt, fopt, optimizer_stats.run(i)] = ro_sem(fitnessfct, N, lb, ub, delta, stopeval, epsilon, param);
			optimizer_stats.run(i).name = optimizer_name;
		end
		save(optimizer_stats_file, 'optimizer_stats');
	else
		load(optimizer_stats_file);
		disp(['file: ', optimizer_stats_file, ' loaded'])
	end
	stat.algorithm(optimizer_index) = optimizer_stats;

	optimizer_stats = [];
	optimizer_name = ['MEM'];
	optimizer_index = 2;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	if (~exist(optimizer_stats_file, 'file'))
		for i = 1 : runs
			disp([optimizer_name, ': test run: ' int2str(i)])
			param.m = 10;
			[xopt, fopt, optimizer_stats.run(i)] = ro_mem(fitnessfct, N, lb, ub, delta, stopeval, epsilon, param);
			optimizer_stats.run(i).name = optimizer_name;
		end
		save(optimizer_stats_file, 'optimizer_stats');
	else
		load(optimizer_stats_file);
		disp(['file: ', optimizer_stats_file, ' loaded'])
	end
	stat.algorithm(optimizer_index) = optimizer_stats;

	optimizer_stats = [];
	optimizer_name = ['Normal'];
	optimizer_index = 3;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	if (~exist(optimizer_stats_file, 'file'))
		for i = 1 : runs
			disp([optimizer_name, ': test run: ' int2str(i)])
			param.m = 100;
			param.nb_select = @lhsu_reference_nb_select_single_candidate;
			param.kriging_method = 'normal';
			[xopt, fopt, optimizer_stats.run(i)] = ro_cmaes_kriging(fitnessfct, N, lb, ub, delta, stopeval, epsilon, param);
			optimizer_stats.run(i).name = optimizer_name;
		end
		save(optimizer_stats_file, 'optimizer_stats');
	else
		load(optimizer_stats_file);
		disp(['file: ', optimizer_stats_file, ' loaded'])
	end
	stat.algorithm(optimizer_index) = optimizer_stats;

	optimizer_stats = [];
	optimizer_name = ['Mahal'];
	optimizer_index = 4;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	if (~exist(optimizer_stats_file, 'file'))
		for i = 1 : runs
			disp([optimizer_name, ': test run: ' int2str(i)])
			param.m = 100;
			param.nb_select = @lhsu_reference_nb_select;
			param.kriging_method = 'mahalanobis';
			[xopt, fopt, optimizer_stats.run(i)] = ro_cmaes_kriging(fitnessfct, N, lb, ub, delta, stopeval, epsilon, param);
			optimizer_stats.run(i).name = optimizer_name;
		end
		save(optimizer_stats_file, 'optimizer_stats');
	else
		load(optimizer_stats_file);
		disp(['file: ', optimizer_stats_file, ' loaded'])
	end
	stat.algorithm(optimizer_index) = optimizer_stats;

end


function [] = run_analysis(fitnessfct, N, lb, ub, delta, stopeval, runs)

	disp(['Test function: ' func2str(fitnessfct)])
	algorithms = 4;

	optimizer_name = ['SEM'];
	optimizer_index = 1;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	stat.algorithm(optimizer_index).name = optimizer_name;
	load(optimizer_stats_file);
	disp(['file: ', optimizer_stats_file, ' loaded'])
	for i = 1 : runs
		stat.algorithm(optimizer_index).run(i).histx = optimizer_stats.run(i).histx;
	end

	optimizer_name = ['MEM'];
	optimizer_index = 2;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	stat.algorithm(optimizer_index).name = optimizer_name;
	load(optimizer_stats_file);
	disp(['file: ', optimizer_stats_file, ' loaded'])
	for i = 1 : runs
		stat.algorithm(optimizer_index).run(i).histx = optimizer_stats.run(i).histx;
	end

	optimizer_name = ['Normal'];
	optimizer_index = 3;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	stat.algorithm(optimizer_index).name = optimizer_name;
	load(optimizer_stats_file);
	disp(['file: ', optimizer_stats_file, ' loaded'])
	for i = 1 : runs
		stat.algorithm(optimizer_index).run(i).histx = optimizer_stats.run(i).histx;
		stat.algorithm(optimizer_index).run(i).approximation_error = optimizer_stats.run(i).approximation_error;
	end

	optimizer_name = ['Mahal'];
	optimizer_index = 4;
	optimizer_stats_file = [func2str(fitnessfct), '_', int2str(N), 'D_', optimizer_name, '.mat'];
	stat.algorithm(optimizer_index).name = optimizer_name;
	load(optimizer_stats_file);
	disp(['file: ', optimizer_stats_file, ' loaded'])
	for i = 1 : runs
		stat.algorithm(optimizer_index).run(i).histx = optimizer_stats.run(i).histx;
		stat.algorithm(optimizer_index).run(i).approximation_error = optimizer_stats.run(i).approximation_error;
	end

	% Normal vs Mahal error plots
	maxgens = Inf;
	for i = 1:runs
		if (length(stat.algorithm(3).run(i).approximation_error) < maxgens)
			maxgens = length(stat.algorithm(3).run(i).approximation_error);
		end
		if (length(stat.algorithm(4).run(i).approximation_error) < maxgens)
			maxgens = length(stat.algorithm(4).run(i).approximation_error);
		end
	end
	error_data_normal = zeros(runs, maxgens);
	error_data_mahal = zeros(runs, maxgens);
	for i=1:runs
		error_data_normal(i,:) = stat.algorithm(3).run(i).approximation_error(1:maxgens);
		error_data_mahal(i,:) = stat.algorithm(4).run(i).approximation_error(1:maxgens);
	end
	clf
	plot([1:maxgens], mean(error_data_normal, 1), '-ok', 'linewidth', 2, 'MarkerSize', 8)
	hold on
	plot([1:maxgens], mean(error_data_mahal, 1), '-.xk', 'linewidth', 2, 'MarkerSize', 8)
	xlabel('generations','FontWeight','Bold', 'FontSize', 20)
	ylabel(['fitness approximation error'],'FontWeight','Bold', 'FontSize', 20)
	legend('Normal', 'Mahal')
	set(gca,'FontSize',20)

	set(gcf,'Position',[0,0,790,640])
	drawnow()
	set(gcf,'PaperPositionMode','auto')
	saveas(gcf, ['results_', func2str(fitnessfct), '_' ,int2str(N), 'D_approximation_error.png'])

	plot_resolution = (stopeval / 20);
	m = 200;
	for j = 1 : algorithms
		for i = 1 : runs
			stat.algorithm(j).run(i).histf_eff = Inf(1, stopeval);
			stat.algorithm(j).run(i).histf_raw = Inf(1, stopeval);
			for k = 1 : plot_resolution : stopeval
				stat.algorithm(j).run(i).histf_eff(k) = approx_f_eff(stat.algorithm(j).run(i).histx(:,k)', fitnessfct, @lhsu, delta, m);
				stat.algorithm(j).run(i).histf_raw(k) = feval(fitnessfct, stat.algorithm(j).run(i).histx(:,k)');
			end
			k = stopeval;
			stat.algorithm(j).run(i).histf_eff(k) = approx_f_eff(stat.algorithm(j).run(i).histx(:,k)', fitnessfct, @lhsu, delta, m);
			stat.algorithm(j).run(i).histf_raw(k) = feval(fitnessfct, stat.algorithm(j).run(i).histx(:,k)');

			stat.algorithm(j).histf_eff(:,i) = stat.algorithm(j).run(i).histf_eff(1:stopeval);
			stat.algorithm(j).histf_raw(:,i) = stat.algorithm(j).run(i).histf_raw(1:stopeval);
		end
	end

	clf
	colors = {'-k', '-ok', '-sk', '-+k', '-^k', '-*k', '-dk', '-.r', '-.g', '-.m', '-.c', '-.k'};
	legend_strings = java_array('java.lang.String', algorithms);
	label_strings = java_array('java.lang.String', algorithms);
	for i = 1 : algorithms
		legend_strings(i) = java.lang.String(regexprep(stat.algorithm(i).name, '\_', '\\_'));
		label_strings(i) = java.lang.String(stat.algorithm(i).name);
	end

	% Convergence plots
	for i = 1 : algorithms
		x_data = [1:plot_resolution:stopeval];
		f_data = mean(stat.algorithm(i).histf_eff(1:plot_resolution:stopeval,:), 2);

		x_data(end+1) = stopeval;
		f_data(end+1) = mean(stat.algorithm(i).histf_eff(stopeval,:));

		%std_data = std(stat.algorithm(i).histf_eff(1:plot_resolution:stopeval,:), [], 2);
		plot(x_data, f_data, char(colors(i)), 'linewidth', 2, 'MarkerSize', 8);
		if (i == 1)
			hold on
		end
	end
	hold off
	legend(gca, cell(legend_strings), 'FontSize', 20)
	grid on
	xlabel('evaluations','FontWeight','Bold', 'FontSize', 20)
	ylabel('robust fitness approximation','FontWeight','Bold', 'FontSize', 20)
	set(gca,'FontSize',16)

	set(gcf,'Position',[0,0,790,640])
	drawnow()
	set(gcf,'PaperPositionMode','auto')
	saveas(gcf, ['results_', func2str(fitnessfct), '_' ,int2str(N), 'D_convergence.png'])

	% Boxplots
	clf
	f_data = [];
	for i = 1 : algorithms
		f_data(:,i) = stat.algorithm(i).histf_eff(stopeval,:);
		plot(i * ones(runs,1), stat.algorithm(i).histf_eff(stopeval,:), 'xk', 'linewidth', 2, 'MarkerSize', 8)
		if (i == 1)
			hold on
		end
	end
	h = boxplot(f_data, 'labels', cell(label_strings), 'notch','on');
	hold off
	for ih = 1 : 6
		set(h(ih,:),'LineWidth',2);
		set(h(ih,:),'Color','k');
	end
	ylabel(['fitness approximation after ' int2str(stopeval) ' evaluations'],'FontWeight','Bold', 'FontSize', 20)
	set(gca,'FontSize', 20)
	grid on

	set(gcf,'Position',[0,0,790,640])
	drawnow()
	set(gcf,'PaperPositionMode','auto')
	saveas(gcf, ['results_', func2str(fitnessfct), '_' ,int2str(N), 'D_quality.png'])

	% Statistics
	mean(f_data, 1)
	std(f_data, [], 1)
	median(f_data, 1)

	merged_f_data = reshape(f_data, [], 1);
	source_data = reshape(repmat([1:algorithms], runs, 1), [], 1);
	[sort_merged_f_data, sort_index] = sort(merged_f_data);
	for i = 1 : algorithms
		rank_sum(i) = sum(find(source_data(sort_index) == i));
	end
	rank_sum

end
